import unittest
import surfex
from datetime import datetime


class ObsTest(unittest.TestCase):

    def setUp(self):
        self.testdata = "testdata/"
        self.settings = {
            "t2m": {
                "netatmo_label": {
                    "filepattern": self.testdata + "/20201113T05@mm@01Z_all.json",
                    "varname": "Temperature",
                    "filetype": "netatmo",
                    "lonrange": [9.5, 10.5],
                    "latrange": [59.5, 60.5],
                    "debug": True
                },
                "bufr_label": {
                    "filepattern": self.testdata + "/ob2020111306",
                    "filetype": "bufr",
                    "varname": "airTemperatureAt2M",
                    "unit": "K",
                    "range": 1800,
                    "debug": True
                },
                "frost_label": {
                    "varname": "air_temperature",
                    "unit": "K",
                    "filetype": "frost",
                    "debug": True,
                    "lonrange": [10, 11],
                    "latrange": [59, 60],
                    "level": {
                        "levelType": "height_above_ground",
                        "unit": "m",
                        "value": 2
                    },
                }
            },
            "rh2m": {
                "netatmo_label": {
                    "filepattern": self.testdata + "/20201113T05@mm@01Z_all.json",
                    "varname": "Humidity",
                    "filetype": "netatmo",
                    "lonrange": [9.5, 10.5],
                    "latrange": [59.5, 60.5],
                    "debug": True
                },
                "bufr_label": {
                    "filepattern": self.testdata + "/ob2020111306",
                    "filetype": "bufr",
                    "varname": "relativeHumidityAt2M",
                    "unit": "1",
                    "range": 1800,
                    "debug": True
                },
                "frost_label": {
                    "varname": "relative_humidity",
                    "unit": "1",
                    "filetype": "frost",
                    "debug": True,
                    "lonrange": [10, 11],
                    "latrange": [59, 60],
                    "level": {
                        "levelType": "height_above_ground",
                        "unit": "m",
                        "value": 2
                    },
                }
            },
            "sd": {
                "bufr_label": {
                    "filepattern": self.testdata + "/ob2020111306",
                    "filetype": "bufr",
                    "varname": "totalSnowDepth",
                    "unit": "m",
                    "range": 1800,
                    "debug": True
                },
                "frost_label": {
                    "varname": "surface_snow_thickness",
                    "unit": "m",
                    "filetype": "frost",
                    "debug": True,
                    "lonrange": [10, 11],
                    "latrange": [59, 60],
                }
            }
        }

    def test_obs_t2m(self):

        an_time = datetime(2020, 11, 13, 6)
        settings = self.settings["t2m"]
        print(settings)
        datasources = surfex.get_datasources(an_time, settings)
        for d in datasources:
            ft = settings[d.label]["filetype"]
            d.write_json_file("unittest_" + ft + "_t2m.json", indent=2)
            print(d.size)

    def test_obs_rh2m(self):

        an_time = datetime(2020, 11, 13, 6)
        settings = self.settings["rh2m"]
        print(settings)
        datasources = surfex.get_datasources(an_time, settings)
        for d in datasources:
            ft = settings[d.label]["filetype"]
            d.write_json_file("unittest_" + ft + "_rh2m.json", indent=2)
            print(d.size)

    def test_obs_sd(self):

        an_time = datetime(2020, 11, 13, 6)
        settings = self.settings["sd"]
        print(settings)
        datasources = surfex.get_datasources(an_time, settings)
        for d in datasources:
            ft = settings[d.label]["filetype"]
            d.write_json_file("unittest_" + ft + "_sd.json", indent=2)
            print(d.size)

    def test_bufr2json(self):
        argv = [
            "-v", "airTemperatureAt2M",
            "-b", self.testdata + "/ob2020111306",
            "-o", "unittest_bufr2json_t2m.json",
            "-dtg", "2020111306",
            "-range", "1800"
        ]
        kwargs = surfex.parse_args_bufr2json(argv)
        surfex.run_bufr2json(**kwargs)



from datetime import datetime, timedelta
import surfex
import json


class TimeSeries(object):
    def __init__(self, times, values, lons, lats, stids, stids_file=None, varname="NA", debug=False):
        self.times = times
        self.values = values
        self.lons = lons
        self.lats = lats
        self.stids = stids
        self.varname = varname
        self.index_pos = {}

        for i in range(0, len(self.lons)):
            lon = self.lons[i]
            lat = self.lats[i]
            pos = surfex.Observation.format_lon(lon) + ":" + surfex.Observation.format_lat(lat)
            self.index_pos.update({pos: i})

        if stids_file is not None:
            self.stids = self.update_stids_from_file(stids_file)

    def update_stids_from_file(self, filename):
        stids = self.stids
        ids_from_file = json.load(open(filename, "r"))
        for stid in ids_from_file:
            lon = ids_from_file[stid]["lon"]
            lat = ids_from_file[stid]["lat"]
            pos = surfex.Observation.format_lon(lon) + ":" + surfex.Observation.format_lat(lat)
            if pos in self.index_pos:
                stids[self.index_pos[pos]] = stid
        return stids

    def write_json(self, filename, indent=None):
        data = {}
        for i in range(0, len(self.times)):
            data.update({
                self.times[i].strftime("%Y%m%d%H%M%S"): {
                    "values": self.values[i].tolist()
                }
            })
        data = {
            "lons": self.lons.tolist(),
            "lats": self.lats.tolist(),
            "stids": self.stids,
            "varname": self.varname,
            "data": data
        }
        json.dump(data, open(filename, "w"), indent=indent)


class TimeSeriesFromJson(TimeSeries):
    def __init__(self, filename, starttime=None, endtime=None, interval=None, lons=None, lats=None, debug=False):
        data = json.load(open(filename, "r"))
        times = []
        values = []
        ts_lons = data["lons"]
        ts_lats = data["lats"]
        ts_stids = data["stids"]
        mask = []
        lons1 = []
        lats1 = []
        stids1 = []
        for i in range(0, len(ts_lons)):
            if lons is not None and lats is not None:
                lon1 = surfex.Observation.format_lon(float(ts_lons[i]))
                lat1 = surfex.Observation.format_lat(float(ts_lats[i]))
                if len(lons) != len(lats):
                    raise Exception("Mismach in longitudes and latitudes")
                for j in range(0, len(lons)):
                    lon = surfex.Observation.format_lon(float(lons[j]))
                    lat = surfex.Observation.format_lat(float(lats[j]))
                    if lon == lon1 and lat == lat1:
                        mask.append(i)
                        lons1.append(ts_lons[i])
                        lats1.append(ts_lats[i])
                        stids1.append(ts_stids[i])
                        break
            else:
                mask.append(i)
                lons1.append(ts_lons[i])
                lats1.append(ts_lats[i])
                stids1.append(ts_stids[i])

        if lons is not None and lats is not None:
            if len(mask) != len(lons):
                print(ts_lons, ts_lats)
                print(lons, lats)
                raise Exception("You asked for " + str(len(lons) - len(mask)) + " position(s) not in the file")

        varname = data["varname"]
        validtime = None
        if starttime is not None:
            validtime = starttime

        for dt in data["data"]:
            add = True
            this_time = datetime.strptime(dt, "%Y%m%d%H%M%S")
            if starttime is not None and this_time < starttime:
                add = False
            if endtime is not None and this_time > endtime:
                add = False
            if interval is not None and validtime is not None and this_time != validtime:
                add = False

            if add:
                times.append(this_time)
                this_values = []
                for i in range(0, len(mask)):
                    this_values.append(data["data"][dt]["values"][mask[i]])
                values.append(this_values)
            else:
                print("Skip this time ", this_time)

            if validtime is not None:
                if interval is not None:
                    validtime = validtime + timedelta(seconds=interval)
        TimeSeries.__init__(self, times, values, lons1, lats1, stids1, varname=varname, debug=debug)


class TimeSeriesFromConverter(TimeSeries):

    """
    Time-Series object
    """

    def __init__(self, var, fileformat, conf, geo, converter, start, end, interval=3600, geo_in=None, cache=None,
                 stids_file=None, debug=False):

        validtime = start
        basetime = start
        defs = {}

        converter = surfex.Converter(converter, validtime, defs, conf[var][fileformat]["converter"], fileformat,
                                     basetime)
        times = []
        values = []
        # Loop output time steps
        this_time = start
        while this_time <= end:
            # Write for each time step
            print("Creating time series for: " + this_time.strftime('%Y%m%d%H') + " time_step:" + str(this_time))
            values.append(converter.read_time_step(geo, this_time, cache, geo_in=geo_in))
            times.append(this_time)

            this_time = this_time + timedelta(seconds=interval)
            if cache is not None:
                cache.clean_fields(this_time)

        if stids_file is not None:
            stids = surfex.Observation.get_stid_from_stationlist(stids_file, geo.lonlist, geo.latlist)
        else:
            stids = ["NA"] * geo.nlons

        TimeSeries.__init__(self, times, values, geo.lonlist, geo.latlist, stids, stids_file=stids_file, varname=var,
                            debug=debug)



from abc import ABC, abstractmethod
import math
import pyproj
import numpy as np
import surfex
import os


class Geo(object):
    def __init__(self, npoints, nlons, nlats, lons, lats, from_json=None, proj=None):
        can_interpolate = False
        if type(lons) != np.ndarray or type(lats) != np.ndarray:
            raise Exception("Longitudes and latitudes must be numpy nd arrays")
        self.proj = proj
        self.npoints = npoints
        self.nlons = nlons
        self.nlats = nlats
        self.lonlist = lons.flatten()
        self.latlist = lats.flatten()
        self.lonrange = None
        self.latrange = None
        if lons.shape[0] > 0 and lats.shape[0] > 0:
            if self.npoints != self.nlons and self.npoints != self.nlats:
                # Make 2D array
                can_interpolate = True
                self.lons = np.reshape(self.lonlist, [self.nlons, self.nlats])
                self.lats = np.reshape(self.latlist, [self.nlons, self.nlats])
            self.lonrange = [np.min(lons), np.max(lons)]
            self.latrange = [np.min(lats), np.max(lats)]
        self.can_interpolate = can_interpolate
        self.json = from_json

    def identifier(self):
        f_lon = ""
        l_lon = ""
        if self.lonlist is not None:
            f_lon = str(round(float(self.lonlist[0]), 2))
            l_lon = str(round(float(self.lonlist[-1]), 2))
        f_lat = ""
        l_lat = ""
        if self.latlist is not None:
            f_lat = str(round(float(self.latlist[0]), 2))
            l_lat = str(round(float(self.latlist[-1]), 2))

        tag = ":" + str(self.npoints) + ":" + str(self.nlons) + ":" + str(self.nlats) + ":" + f_lon + ":" + l_lon +\
              ":" + f_lat + ":" + l_lat + ":"
        tag = tag.replace(" ", "")
        return tag

    def is_identical(self, geo_to_check):
        if self.identifier() == geo_to_check.identifier():
            print("Geometries are identical")
            return True
        else:
            return False


class SurfexGeo(ABC, Geo):
    def __init__(self, proj, npoints, nlons, nlats, lons, lats, from_json):
        self.mask = None
        self.proj = proj
        Geo.__init__(self, npoints, nlons, nlats, lons, lats, from_json=from_json, proj=proj)

    @abstractmethod
    def update_namelist(self, nml):
        return NotImplementedError


class ConfProj(SurfexGeo):
    def __init__(self, from_json):
        self.cgrid = "CONF PROJ"
        domain_dict = surfex.BaseNamelist.lower_case_namelist_dict(from_json)

        self.ilone = None
        self.ilate = None
        if "nam_conf_proj_grid" in domain_dict:
            if "nimax" and "njmax" and "xloncen" and "xlatcen" and "xdx" and "xdy" in domain_dict["nam_conf_proj_grid"]:
                self.nimax = domain_dict["nam_conf_proj_grid"]["nimax"]
                self.njmax = domain_dict["nam_conf_proj_grid"]["njmax"]
                self.xloncen = domain_dict["nam_conf_proj_grid"]["xloncen"]
                self.xlatcen = domain_dict["nam_conf_proj_grid"]["xlatcen"]
                self.xdx = domain_dict["nam_conf_proj_grid"]["xdx"]
                self.xdy = domain_dict["nam_conf_proj_grid"]["xdy"]
                if "ilone" in domain_dict["nam_conf_proj_grid"]:
                    self.ilone = domain_dict["nam_conf_proj_grid"]["ilone"]
                if "ilate" in domain_dict["nam_conf_proj_grid"]:
                    self.ilate = domain_dict["nam_conf_proj_grid"]["ilate"]
            else:
                raise KeyError("Missing keys1")
        else:
            raise KeyError("Missing key2")

        if "nam_conf_proj" in domain_dict:
            if "xlon0" and "xlat0" in domain_dict["nam_conf_proj"]:
                self.xlon0 = domain_dict["nam_conf_proj"]["xlon0"]
                self.xlat0 = domain_dict["nam_conf_proj"]["xlat0"]
            else:
                raise KeyError("Missing keys3")
        else:
            raise KeyError("Missing key4")

        earth = 6.37122e+6
        proj_string = "+proj=lcc +lat_0=" + str(self.xlat0) + " +lon_0=" + str(self.xlon0) + " +lat_1=" + \
                      str(self.xlat0) + " +lat_2=" + str(self.xlat0) + " +units=m +no_defs +R=" + str(earth)

        proj = pyproj.CRS.from_string(proj_string)
        wgs84 = pyproj.CRS.from_string("EPSG:4326")

        xloncen, xlatcen = \
            pyproj.Transformer.from_crs(wgs84, proj, always_xy=True).transform(self.xloncen, self.xlatcen)

        x0 = xloncen - (0.5 * (float(self.nimax) - 1.) * self.xdx)
        y0 = xlatcen - (0.5 * (float(self.njmax) - 1.) * self.xdy)
        x = np.arange(x0, x0 + (self.nimax * self.xdx), self.xdx)
        y = np.arange(y0, y0 + (self.njmax * self.xdy), self.xdy)
        xv, yv = np.meshgrid(x, y)
        lons, lats = pyproj.Transformer.from_crs(proj, wgs84, always_xy=True).transform(xv, yv)

        npoints = self.nimax * self.njmax
        SurfexGeo.__init__(self, proj, npoints, self.nimax, self.njmax, np.reshape(lons, [npoints], order="F"),
                           np.reshape(lats, [npoints], order="F"), from_json)

    def update_namelist(self, nml):
        if self.ilate is None or self.ilate is None:
            nml.update({
                "nam_pgd_grid": {
                    "cgrid": self.cgrid
                },
                "nam_conf_proj": {
                    "xlon0": self.xlon0,
                    "xlat0": self.xlat0,
                    "xrpk": math.sin(math.radians(self.xlat0)),
                    "xbeta": 0},
                "nam_conf_proj_grid": {
                    "xlatcen": self.xlatcen,
                    "xloncen": self.xloncen,
                    "nimax": self.nimax,
                    "njmax": self.njmax,
                    "xdx": self.xdx,
                    "xdy": self.xdy
                }
            })
        else:
            nml.update({
                "nam_pgd_grid": {
                    "cgrid": self.cgrid
                },
                "nam_conf_proj": {
                    "xlon0": self.xlon0,
                    "xlat0": self.xlat0,
                    "xrpk": math.sin(math.radians(self.xlat0)),
                    "xbeta": 0},
                "nam_conf_proj_grid": {
                    "ilone": self.ilone,
                    "ilate": self.ilate,
                    "xlatcen": self.xlatcen,
                    "xloncen": self.xloncen,
                    "nimax": self.nimax,
                    "njmax": self.njmax,
                    "xdx": self.xdx,
                    "xdy": self.xdy
                }
            })
        return nml


class LonLatVal(SurfexGeo):
    def __init__(self, from_json):
        self.cgrid = "LONLATVAL"
        domain_dict = surfex.BaseNamelist.lower_case_namelist_dict(from_json)

        if "nam_lonlatval" in domain_dict:
            if "xx" and "xy" and "xdx" and "xdy" in domain_dict["nam_lonlatval"]:
                self.xx = domain_dict["nam_lonlatval"]["xx"]
                self.xy = domain_dict["nam_lonlatval"]["xy"]
                self.xdx = domain_dict["nam_lonlatval"]["xdx"]
                self.xdy = domain_dict["nam_lonlatval"]["xdy"]
                proj4 = "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84"
                proj = pyproj.CRS.from_string(proj4)
                SurfexGeo.__init__(self, proj, len(self.xx), len(self.xx), len(self.xy), np.asarray(self.xx),
                                   np.asarray(self.xy), from_json)
                self.can_interpolate = False
            else:
                print("Missing keys")
                raise KeyError
        else:
            print("Missing key")
            raise KeyError

    def update_namelist(self, nml):
        nml.update({
            "nam_pgd_grid": {
                "cgrid": self.cgrid
            },
            "nam_lonlatval": {
                "xx": self.xx,
                "xy": self.xy,
                "xdx": self.xdx,
                "xdy": self.xdy
            }
        })
        return nml


class Cartesian(SurfexGeo):
    def __init__(self, from_json):
        self.cgrid = "CARTESIAN"
        domain_dict = surfex.BaseNamelist.lower_case_namelist_dict(from_json)

        if "nam_cartesian" in domain_dict:
            if "xlat0" and "xlon0" and "nimax" and "njmax" and "xdx" and "xdy" in domain_dict["nam_cartesian"]:
                self.xlat0 = domain_dict["nam_cartesian"]["xlat0"]
                self.xlon0 = domain_dict["nam_cartesian"]["xlon0"]
                self.nimax = domain_dict["nam_cartesian"]["nimax"]
                self.njmax = domain_dict["nam_cartesian"]["njmax"]
                self.xdx = domain_dict["nam_cartesian"]["xdx"]
                self.xdy = domain_dict["nam_cartesian"]["xdy"]
                proj = None
                # proj, npoints, nlons, nlats, lons, lats
                SurfexGeo.__init__(self, proj, self.nimax * self.njmax, self.nimax, self.njmax, np.asarray([]),
                                   np.asarray([]), from_json)
                self.can_interpolate = False
            else:
                print("Missing keys")
                raise KeyError
        else:
            print("Missing key")
            raise KeyError

    def update_namelist(self, nml):
        print(nml)
        nml.update({
            "nam_pgd_grid": {
                "cgrid": self.cgrid
            },
            "nam_cartesian": {
                "xlat0": self.xlat0,
                "xlon0": self.xlon0,
                "nimax": self.nimax,
                "njmax": self.njmax,
                "xdx": self.xdx,
                "xdy": self.xdy
            }
        })
        return nml


class LonLatReg(SurfexGeo):
    def __init__(self, from_json):
        self.cgrid = "LONLAT REG"
        domain_dict = surfex.BaseNamelist.lower_case_namelist_dict(from_json)

        if "nam_lonlat_reg" in domain_dict:
            if "xlonmin" and "xlonmax" and "xlatmin" and "xlatmax" and "nlon" and "nlat" \
                    in domain_dict["nam_lonlat_reg"]:
                self.xlonmin = domain_dict["nam_lonlat_reg"]["xlonmin"]
                self.xlonmax = domain_dict["nam_lonlat_reg"]["xlonmax"]
                self.xlatmin = domain_dict["nam_lonlat_reg"]["xlatmin"]
                self.xlatmax = domain_dict["nam_lonlat_reg"]["xlatmax"]
                self.nlon = domain_dict["nam_lonlat_reg"]["nlon"]
                self.nlat = domain_dict["nam_lonlat_reg"]["nlat"]
            else:
                print("Missing keys")
                raise KeyError
        else:
            print("Missing key")
            raise KeyError

        proj_string = "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84"
        proj = pyproj.CRS.from_string(proj_string)
        lons = []
        lats = []
        if self.nlon == 0 or self.nlat == 0:
            raise ZeroDivisionError

        dlon = (self.xlonmax - self.xlonmin) / (self.nlon - 1)
        dlat = (self.xlatmax - self.xlatmin) / (self.nlat - 1)
        print(dlon, dlat)
        for j in range(0, self.nlat):
            for i in range(0, self.nlon):
                lons.append(self.xlonmin + i * dlon)
                lats.append(self.xlatmin + j * dlat)

        # proj, npoints, nlons, nlats, lons, lats
        SurfexGeo.__init__(self, proj, self.nlon * self.nlat, self.nlon, self.nlat, np.asarray(lons), np.asarray(lats),
                           from_json)

    def update_namelist(self, nml):
        nml.update({
            "nam_pgd_grid": {
                "cgrid": self.cgrid
            },
            "nam_lonlat_reg": {
                "xlonmin": self.xlonmin,
                "xlonmax": self.xlonmax,
                "xlatmin": self.xlatmin,
                "xlatmax": self.xlatmax,
                "nlon": self.nlon,
                "nlat": self.nlat
            }
        })
        return nml


class IGN(SurfexGeo):
    def __init__(self, from_json, recreate=False):
        self.cgrid = "IGN"
        domain_dict = surfex.BaseNamelist.lower_case_namelist_dict(from_json)

        if "nam_ign" in domain_dict:
            if "clambert" and "npoints" and "xx" and "xy" and "xdx" and "xdy" and "xx_llcorner" and "xy_llcorner"  \
                    and "xcellsize" and "ncols" and "nrows" in domain_dict["nam_ign"]:

                self.clambert = domain_dict["nam_ign"]["clambert"]
                npoints = domain_dict["nam_ign"]["npoints"]
                self.xx = domain_dict["nam_ign"]["xx"]
                self.xy = domain_dict["nam_ign"]["xy"]
                self.xdx = domain_dict["nam_ign"]["xdx"]
                self.xdy = domain_dict["nam_ign"]["xdy"]
                self.xx_llcorner = domain_dict["nam_ign"]["xx_llcorner"]
                self.xy_llcorner = domain_dict["nam_ign"]["xy_llcorner"]
                self.xcellsize = domain_dict["nam_ign"]["xcellsize"]
                self.ncols = domain_dict["nam_ign"]["ncols"]
                self.nrows = domain_dict["nam_ign"]["nrows"]
            else:
                print("Missing keys")
                raise KeyError
        else:
            print("Missing key")
            raise KeyError

        if self.clambert == 7:
            proj4 = "+proj=lcc +lat_0=63.5 +lon_0=15.0 +lat_1=63.5 +lat_2=63.5 +no_defs +R=6.37122e+6"
            self.xloncen = 17
            self.xlatcen = 63.
            self.xlon0 = 15
            self.xlat0 = 63.5
        else:
            raise NotImplementedError

        proj = pyproj.CRS.from_string(proj4)
        wgs84 = pyproj.CRS.from_string("EPSG:4326")

        pxall = self.get_coord(self.xx, self.xdx, "x", recreate)
        pyall = self.get_coord(self.xy, self.xdy, "y", recreate)
        self.mask = self.ign_mask(pxall, pyall, self.xx, self.xy, recreate)

        # proj, npoints, nlons, nlats, lons, lats
        lons = []
        lats = []
        for i in range(0, npoints):
            lon, lat = pyproj.Transformer.from_crs(proj, wgs84, always_xy=True).transform(self.xx[i], self.xy[i])
            lons.append(lon)
            lats.append(lat)

        SurfexGeo.__init__(self, proj, npoints, npoints, npoints, np.asarray(lons), np.asarray(lats), from_json)
        self.can_interpolate = False

    @staticmethod
    def get_coord(pin, pdin, coord, recreate=False):

        pout = []
        cache = "/tmp/." + coord + "_cached"
        if os.path.isfile(cache) and not recreate:
            f = open(cache)
            cached_coord = f.read().splitlines()
            f.close()
            for i in range(0, len(cached_coord)):
                pout.append(float(cached_coord[i]))
            return pout

        zdout = []
        ksize = 0
        if len(pin) > 0:
            zdout.append(float(pdin[0]) / 2.)
            pout.append(pin[0])
            ksize = 1
            if len(pin) > 1:
                ksize = 2
                pout.append(pin[0] - pdin[0])
                zdout.append(0.)
            if len(pin) > 2:
                ksize = 3
                pout.append(pin[0] + pdin[0])
                zdout.append(0.)

        # print ksize
        for i in range(0, len(pin)):
            for j in range(0, ksize):
                # print i,j,len(pin),ksize,pout[j],pin[i]
                if pout[j] == pin[i]:
                    break
                if j == ksize - 1:
                    ksize = ksize + 1
                    pout.append(pin[i])
                    zdout.append(float(pdin[i]) / 2.)

            # Mesh constrains
            for j in range(0, ksize):
                # print i, j, len(pin), ksize, pout[j], pin[i]
                if pout[j] < pin[i] and (pout[j] + zdout[j]) >= (pin[i] - pdin[i]):
                    break
                if j == ksize - 1:
                    ksize = ksize + 1
                    pout.append(pin[i] - pdin[i])
                    zdout.append(0.)

            for j in range(0, ksize):
                if pout[j] > pin[i] and (pout[j] - zdout[j]) <= (pin[i] + pdin[i]):
                    break
                if j == ksize - 1:
                    ksize = ksize + 1
                    pout.append(pin[i] + pdin[i])
                    zdout.append(0.)

        # Sort pout
        pout = sorted(pout)

        f = open(cache, "w")
        for i in range(0, len(pout)):
            f.write(str(pout[i]) + "\n")
        print("Cached coordinates for : ", coord)
        f.close()

        return pout

    @staticmethod
    def ign_mask(pxall, pyall, xx, yy, recreate):
        mask = []

        cache = "/tmp/.mask"
        if os.path.isfile(cache) and not recreate:
            f = open(cache)
            cached_mask = f.read().splitlines()
            f.close()
            for i in range(0, len(cached_mask)):
                mask.append(int(cached_mask[i]))

            if len(mask) != len(xx) or len(mask) != len(yy):
                raise Exception("Cached mask mismatch! ", len(mask), len(xx), len(yy))

            return mask

        print("Creating mask. This takes time:")
        count = -1

        for i in range(0, len(pxall)):
            for j in range(0, len(pyall)):

                count = count + 1
                for k in range(0, len(xx)):
                    if xx[k] == pxall[i] and yy[k] == pyall[j]:
                        # print i,j,k,l,xx[k],pxall[i],yy[k],pyall[j]
                        mask.append(count)
                        break

            print(i, "/", len(pxall))

        # Cache mask for later use
        # if len(mask) != len(xx) or len(mask) != len(yy): print "Mask mismatch! ", len(mask), len(xx), len(yy); exit(1)
        f = open(cache, "w")
        for i in range(0, len(mask)):
            f.write(str(mask[i])+"\n")

        # f.write("mask="+str(mask)+"\n")
        print("Created mask: ", mask)
        f.close()
        return mask

    def update_namelist(self, nml):
        nml.update({
            "nam_pgd_grid": {
                "cgrid": self.cgrid
            },
            "nam_ign": {
                "clambert": self.clambert,
                "npoints": self.npoints,
                "xx": self.xx,
                "xy": self.xy,
                "xdx": self.xdx,
                "xdy": self.xdy,
                "xx_llcorner": self.xx_llcorner,
                "xy_llcorner": self.xy_llcorner,
                "xcellsize": self.xcellsize,
                "ncols": self.ncols,
                "nrows": self.nrows
            }
        })
        return nml


def get_geo_object(from_json):
    domain_dict = {}
    for key in from_json:
        lower_case_dict = {}
        for key2 in from_json[key]:
            lower_case_dict.update({key2.lower(): from_json[key][key2]})
        domain_dict.update({key.lower(): lower_case_dict})

    if "nam_pgd_grid" in domain_dict:
        if "cgrid" in domain_dict["nam_pgd_grid"]:
            if domain_dict["nam_pgd_grid"]["cgrid"] == "CONF PROJ":
                return ConfProj(from_json)
            elif domain_dict["nam_pgd_grid"]["cgrid"] == "LONLATVAL":
                return LonLatVal(from_json)
            elif domain_dict["nam_pgd_grid"]["cgrid"] == "LONLAT REG":
                return LonLatReg(from_json)
            elif domain_dict["nam_pgd_grid"]["cgrid"] == "IGN":
                return IGN(from_json)
            elif domain_dict["nam_pgd_grid"]["cgrid"] == "CARTESIAN":
                return Cartesian(from_json)
            else:
                raise NotImplementedError
        else:
            print("Missing grid information cgrid")
            raise KeyError
    else:
        print("nam_pgd_grid not set!")
        raise KeyError


def set_domain(settings, domain, hm_mode=False):
    if type(settings) is dict:
        if domain in settings:
            if hm_mode:

                ezone = 11
                if "EZONE" in settings[domain]:
                    ezone = settings[domain]["EZONE"]

                domain_dict = {
                    "nam_pgd_grid": {
                        "cgrid": "CONF PROJ"
                    },
                    "nam_conf_proj": {
                        "xlat0": settings[domain]["LAT0"],
                        "xlon0": settings[domain]["LON0"],
                    },
                    "nam_conf_proj_grid": {
                        "ilone": ezone,
                        "ilate": ezone,
                        "xlatcen": settings[domain]["LATC"],
                        "xloncen": settings[domain]["LONC"],
                        "nimax": settings[domain]["NLON"] - ezone,
                        "njmax": settings[domain]["NLAT"] - ezone,
                        "xdx": settings[domain]["GSIZE"],
                        "xdy": settings[domain]["GSIZE"],
                    }
                }
            else:
                domain_dict = settings[domain]
            return domain_dict
        else:
            print("Domain not found: " + domain)
            raise Exception("Domain not found: " + domain)
    else:
        print("Settings should be a dict")
        raise Exception("Settings should be a dict")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()
    # print(long_description)


def read(rel_path):
    with open(path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


setup(
    name='pysurfex',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=get_version("surfex/__init__.py"),

    description='Python API to SURFEX',
    long_description='Python API to SURFEX',

    # The project's main homepage.
    url='https://github.com/metno/forcing-offline-surfex',

    # Author details
    author='Trygve Aspelien',
    author_email='trygve.aspelien@met.no',

    # Choose your license
    license='BSD-3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
        'Topic :: Scientific/Engineering :: Information Analysis',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: BSD License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    # What does your project relate to?
    keywords='meteorology weather prediction offline surfex forcing',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', '*tests*']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        "numpy",
        "scipy",
        "netCDF4",
        "cfunits",
        "pyproj == 2.2",
        "pyyaml",
        "tomlkit",
        "toml",
        "netCDF4",
        "jsonmerge",
        "datetime",
        "f90nml",
        "cfunits",
        "enum34",
        "requests",
        "json; python_version < '3'",
        "StringIO; python_version < '3'",
        "eccodes",
        "python-csv",
        "db-sqlite3",
        "titanlib >= 0.2.1",
        "gridpp >= 0.5.0.post1"
    ],

    # Not on pypi
    # epygram

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
    #    'dev': ['check-manifest'],
        'test': [
            "coverage",
            "coveralls",
            "pep8",
            "nose",
            "nose-timer"
        ],
        'plot': ['matplotlib'],
        'plot_on_map': ['cartopy'],
    },

    test_suite="surfex.tests",

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_dir={'forcing': 'forcing', "config": "config"},
    package_data={
        'forcing': ['cfg/config.yml', 'cfg/area.yml', 'cfg/user.yml', 'cfg/first_guess.yml'],
        "config": ["cfg/config_exp.toml", "cfg/config_exp_surfex.toml"],
    },
    include_package_data=True,

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('forcing', ['cfg/config.yml','cfg/area.yml','cfg/user.yml'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.

    # entry_points={
    #    'console_scripts': [
    #        'create_forcing=forcing:create_forcing',
    #        'plot_offline=forcing:plot_offline',
    #    ],
    # },
    scripts=[
        'bin/create_forcing',
        'bin/qc2obsmon',
        'bin/create_forcing',
        'bin/FirstGuess4gridpp',
        'bin/gridpp',
        'bin/json-gui2toml',
        'bin/masterodb',
        'bin/merge_json_namelist_settings',
        'bin/merge_toml_files',
        'bin/merge_qc_data',
        'bin/offline',
        'bin/oi2soda',
        'bin/perturbed_offline',
        'bin/pgd',
        'bin/plot_field',
        'bin/plot_timeseries',
        'bin/prep',
        'bin/set_geo_from_obs_set',
        'bin/set_geo_from_stationlist',
        'bin/set_domain',
        'bin/soda',
        'bin/titan',
        'bin/bufr2json'
    ],
)



import unittest
import surfex


class Hm2PysurfexTest(unittest.TestCase):

    @staticmethod
    def test_hm2pysurfex_client():
        argv = [
            "-c", "surfex/cfg/config_exp_surfex.toml",
            "-o", "unittest_config_from_hm.toml",
            "-e", "test/settings/hm_env.json"
        ]
        kwargs = surfex.parse_args_hm2pysurfex(argv)
        surfex.hm2pysurfex(**kwargs)



import surfex
import netCDF4
import numpy as np
import cfunits
import os
import re
from datetime import datetime, date
from enum import Enum


class Netcdf(object):
    def __init__(self, filename):
        self.filename = filename
        print(filename)
        self.file = netCDF4.Dataset(filename, "r")

    def num_height(self, field):
        pass

    def num_time(self, field):
        """
        :param field: 
        :return: (int) length of time dimension 
        """
        pass

    def slice(self, var_name, levels=None, members=None, times=None, xcoords=None, ycoords=None,
              deaccumulate=False, instantanious=0., units=None, lev_from_ind=False):
        """
        Assembles a 5D field in order lon,lat,time,height,ensemble

        Arguments:
            var_name (str): Name of field to retrieve
            levels (list): Height index. If None, return all.
            members (list): Ensemble index. If None, return all.
            times (list): Time index. If None, return all.
            xcoords: X-axis coordinates to subset
            ycoords: Y-axis coordinates to subset
            deaccumulate (bool): Deaccumulate field
            instantanious (float): Scaling factor to make an accumulated value as instantanius
            units (str): CF unit for the variable to be read
            lev_from_ind (bool): level list are indices and not values

        Returns:
         np.array: 5D array with values
        """

        var = NetCDFFileVariable(self.file, var_name)
        if xcoords is not None or ycoords is not None:
            raise Exception("Subsetting of the input dimensions not implemented yet!")

        surfex.util.info("Reading variable "+var.var_name, level=1)
        times_to_read = []
        prev_time_steps = []
        if times is None:
            for i in range(0, var.times.shape[0]):
                times_to_read.append(i)
                if i > 0:
                    prev_time_steps.append(i-1)
                else:
                    prev_time_steps.append(0)
        else:
            if not isinstance(times, (list, tuple)):
                raise Exception("Times must be a list!")
            if isinstance(times[0], date):
                surfex.util.info("Time provided in call as datetime objects", level=2)
                times_in_var = var.datetimes
                for i in range(0, len(times_in_var)):
                    print(i, times_in_var[i], times)
                    for j in range(0, len(times)):
                        # Time steps requested
                        print(times_in_var[i], times[j])
                        if times_in_var[i] == times[j]:
                            times_to_read.append(i)
                            if i > 0:
                                prev_time_steps.append(i-1)
                            else:
                                prev_time_steps.append(0)

            else:
                times_in_var = var.times
                for i in range(0, times_in_var.shape[0]):
                    for j in range(0, len(times)):
                        # Time steps requested
                        if i == times[j]:
                            times_to_read.append(times[j])
                            if i > 0:
                                prev_time_steps.append(i-1)
                            else:
                                prev_time_steps.append(0)

        print("times to read", times_to_read)
        levels_to_read = []
        if levels is None:
            for i in range(0, var.levels.shape[0]):
                levels_to_read.append(i)
        else:
            surfex.util.info("Level provided in call. lev_from_ind=" + str(lev_from_ind), level=2)
            if not isinstance(levels, (list, tuple)):
                raise Exception("Levels must be a list!")
            levels_in_var = var.levels
            for i in range(0, levels_in_var.shape[0]):
                for j in range(0, len(levels)):
                    # print lev_from_ind,i, j, levels_in_var[i], levels[j]
                    if lev_from_ind:
                        if i == levels[j]:
                            levels_to_read.append(i)
                    else:
                        # NB! Round number to avoid round off when matching
                        if round(float(levels_in_var[i]), 5) == round(float(levels[j]), 5):
                            levels_to_read.append(i)

        members_to_read = []
        if members is None:
            for i in range(0, var.members.shape[0]):
                members_to_read.append(i)
        else:
            if not isinstance(members, (list, tuple)):
                raise Exception("Members must be a list!")
            surfex.util.info("Ensemble members provided in call", level=2)
            members_in_var = var.members
            for i in range(0, members_in_var.shape[0]):
                for j in range(0, len(members)):
                    if members_in_var[i] == members[j]:
                        members_to_read.append(i)

            if len(members_to_read) == 0:
                raise Exception("No ensemble members found for " + var.var_name)

        lons = var.lons
        lats = var.lats

        # Dimensions of the "problem"
        dim_x = lons.shape[0]
        dim_y = lats.shape[1]

        geo = surfex.geo.Geo(dim_x * dim_y, dim_x, dim_y, lons, lats)

        dim_t = max(len(times_to_read), 1)
        dim_levels = max(len(levels_to_read), 1)
        dim_members = max(len(members_to_read), 1)

        surfex.util.info("Dimensions in output", level=3)
        surfex.util.info(str(dim_x) + " " + str(dim_y) + " " + str(dim_t) + " " + str(dim_levels) + " " +
                         str(dim_members), level=3)

        lon_ind = slice(0, dim_x, 1)
        lat_ind = slice(0, dim_y, 1)
        dims = []
        prev_dims = []
        types = var.axis_types
        mapping = {}  # Map axis to output axis
        for i in range(0, len(types)):
            if types[i] == Axis.GeoX or types[i] == Axis.Lon:
                dims.append(lon_ind)
                prev_dims.append(lon_ind)
                mapping[0] = i
            elif types[i] == Axis.GeoY or types[i] == Axis.Lat:
                dims.append(lat_ind)
                prev_dims.append(lat_ind)
                mapping[1] = i
            elif types[i] == Axis.Time:
                dims.append(times_to_read)
                prev_dims.append(prev_time_steps)
                mapping[2] = i
            elif var.is_level(types[i]):
                dims.append(levels_to_read)
                prev_dims.append(levels_to_read)
                mapping[3] = i
            elif types[i] == Axis.Realization:
                dims.append(members_to_read)
                prev_dims.append(members_to_read)
                mapping[4] = i
            else:
                raise Exception(str(types[i])+" is not defined!")

        surfex.util.info("Read " + var.var_name + " with dimensions: " + str(dims), level=2)
        if deaccumulate:
            surfex.util.info("Deaccumulate previous dimensions: " + str(prev_dims), level=2)

        print(var.var_name)
        print(dims)
        print(self.file[var.var_name])
        field = self.file[var.var_name][dims]
        if units is not None:
            field = cfunits.Units.conform(field, cfunits.Units(var.units), cfunits.Units(units))

        # Deaccumulation
        if deaccumulate:
            original_field = field
            previous_field = self.file[var.var_name][prev_dims]
            if units is not None:
                previous_field = cfunits.Units.conform(previous_field, cfunits.Units(var.units), cfunits.Units(units))
            field = np.subtract(original_field, previous_field)

        # Create instantanious values
        if instantanious > 0:
            field = np.divide(field, instantanious)

        # Add extra dimensions
        i = 0
        reverse_mapping = []
        for d in range(0, 5):
            if d not in mapping:
                surfex.util.info("Adding dimension " + str(d), level=3)
                field = np.expand_dims(field, len(dims) + i)
                reverse_mapping.append(len(dims) + i)
                i = i + 1
            else:
                reverse_mapping.append(mapping[d])

        # Transpose to 5D array
        surfex.util.info("Transpose to 5D array", level=1)
        field = np.transpose(field, reverse_mapping)

        print("Read netcdf from ", self.filename, "times", times)
        surfex.util.info("Shape of output: "+str(field.shape), level=2)
        return field, geo

    def field(self, var_name, level=None, member=None, validtime=None,  units=None):

        if validtime is None:
            validtime = []
        elif type(validtime) != datetime:
            raise Exception("validime must be a datetime object")
        else:
            validtime = [validtime]

        print(level, member, validtime)
        field, geo_in = self.slice(var_name, levels=level, members=member, times=validtime, units=units)
        # Reshape to fortran 2D style
        field = np.reshape(field, [geo_in.nlons, geo_in.nlats], order="F")
        return field, geo_in

    def points(self, var_name, geo, level=None, member=None, validtime=None,  units=None, interpolation="nearest",
               cache=None):

        """
        Assembles a 5D slice and interpolates it to requested positions

        Arguments:


        Returns:
         np.array: 4D array with inpterpolated values in order pos,time,height,ensemble

        """

        # field4d, geo_in = self.slice(var_name, levels=level, members=member, times=validtime, units=units)
        # field2d = np.transpose(np.reshape(field4d, [geo_in.nlons, geo_in.nlats], order="F"))
        print(level, member, validtime)
        field, geo_in = self.field(var_name, level=level, member=member, validtime=validtime, units=units)
        if interpolation == "nearest":
            surfex.util.info("Nearest neighbour", level=2)
            interpolator = surfex.interpolation.NearestNeighbour(geo_in, geo, cache=cache)
        elif interpolation == "linear":
            surfex.util.info("Linear interpolation", level=2)
            interpolator = surfex.interpolation.Linear(geo_in, geo, cache=cache)
        elif interpolation == "none":
            surfex.util.info("No interpolation", level=2)
            interpolator = surfex.interpolation.NoInterpolation(geo_in, geo, cache=cache)
        else:
            raise NotImplementedError("Interpolation type " + interpolation + " not implemented!")

        field = interpolator.interpolate(field)
        return field, interpolator


class Axis(Enum):
    Undefined = 0
    GeoX = 1
    GeoY = 2
    GeoZ = 3
    Time = 4
    Lon = 5
    Lat = 6
    Pressure = 7
    Height = 8
    ReferenceTime = 9
    Realization = 10
    Hybrid = 11


class NetCDFFileVariable(object):
    def __init__(self, fh, var_name):
        self.file = fh
        self.var_name = var_name

    @property
    def axis_types(self):
        types = []
        if self.var_name not in self.file.variables:
            raise Exception(self.var_name + " is missing in file!")

        if self.file.variables[self.var_name]:
            for i in range(0, len(self.file.variables[self.var_name].dimensions)):
                dim_name = self.file.variables[self.var_name].dimensions[i]
                if dim_name == "longitude" or dim_name == "lon":
                    types.append(Axis.Lon)
                elif dim_name == "x":
                    types.append(Axis.GeoX)
                elif dim_name == "latitude" or dim_name == "lat":
                    types.append(Axis.Lat)
                elif dim_name == "y":
                    types.append(Axis.GeoY)
                elif re.search("height[0-9]*", dim_name):
                    types.append(Axis.Height)
                elif re.search("hybrid[0-9]*", dim_name):
                    types.append(Axis.Hybrid)
                elif re.search("pressure[0-9]*", dim_name):
                    types.append(Axis.Pressure)
                # TODO: GeoZ
                elif dim_name == "ensemble_member":
                    types.append(Axis.Realization)
                elif dim_name == "time":
                    types.append(Axis.Time)
                else:
                    types.append(Axis.Undefined)
        return types

    @property
    def dim_names(self):
        names = []
        if self.file.variables[self.var_name]:
            for i in range(0, len(self.file.variables[self.var_name].dimensions)):
                names.append(self.file.variables[self.var_name].dimensions[i])
        return names

    @property
    def units(self):
        units = None
        if self.file.variables[self.var_name].units:
            units = self.file.variables[self.var_name].units
        return units

    @property
    def lats(self):
        """
        Returns:
           np.array: 2D array of latitudes
        """

        latvals = np.array([])
        axis_types = self.axis_types
        for i in range(0, len(axis_types)):
            if axis_types[i] == Axis.Lat:
                latvals = self.file.variables[self.dim_names[i]]
                surfex.util.warning("Assumed to 2D in (lon,lat) order")
            elif axis_types[i] == Axis.GeoY:
                # TODO: if lat/lon are 1D, create a 2D mesh
                # TODO: Assume the name for now. Must be found in attributes
                latvals = self.file.variables["latitude"]
                latvals = np.transpose(latvals, (1, 0))

        if latvals.shape[0] == 0:
            raise Exception("No latitude found for " + self.var_name)
        return latvals

    @property
    def lons(self):
        """
        Returns:
           np.array: 2D array of longitudes
        """

        lonvals = np.array([])
        axis_types = self.axis_types
        for i in range(0, len(axis_types)):
            if axis_types[i] == Axis.Lon:
                lonvals = self.file.variables[self.dim_names[i]]
            elif axis_types[i] == Axis.GeoX:
                # TODO: if lat/lon are 1D, create a 2D mesh
                # TODO: Assume the name for now. Must be found in attributes
                lonvals = self.file.variables["longitude"]
                lonvals = np.transpose(lonvals, (1, 0))

        if lonvals.shape[0] == 0:
            raise Exception("No longitude found for " + self.var_name)
        return lonvals

    @property
    def datetimes(self):
        """
            Return:
            list()
        """

        times = []
        axis_types = self.axis_types
        for i in range(0, len(axis_types)):
            if axis_types[i] == Axis.Time:
                val = self.file.variables[self.dim_names[i]]
                for t in range(0, len(val)):
                    epochtime = int(cfunits.Units.conform(val[t], cfunits.Units(val.units),
                                                          cfunits.Units("seconds since 1970-01-01 00:00:00")))
                    print(epochtime)
                    dt = datetime.utcfromtimestamp(epochtime)
                    print(dt)
                    times.append(dt)

        if len(times) == 0:
            surfex.util.info("No time found for " + self.var_name, level=2)
        return times

    @property
    def times(self):
        """
            Return:
            np.array: 1D array of times
        """

        times = np.array([])
        axis_types = self.axis_types
        for i in range(0, len(axis_types)):
            if axis_types[i] == Axis.Time:
                times = self.file.variables[self.dim_names[i]]

        if times.shape[0] == 0:
            surfex.util.info("No time found for "+self.var_name, level=2)
        return times

    @property
    def members(self):
        """
            Return:
            np.array: 1D array of ensemble members
        """

        members = np.array([])
        axis_types = self.axis_types
        for i in range(0, len(axis_types)):
            if axis_types[i] == Axis.Realization:
                members = self.file.variables[self.dim_names[i]]

        if members.shape[0] == 0:
            surfex.util.info("No ensemble members found for " + self.var_name, level=2)
        return members

    @property
    def levels(self):
        """
            Return:
            np.array: 1D array of levels
        """

        levels = np.array([])
        axis_types = self.axis_types
        for i in range(0, len(axis_types)):
            if self.is_level(axis_types[i]):
                levels = self.file.variables[self.dim_names[i]]

        if levels.shape[0] == 0:
            surfex.util.info("No levels found for " + self.var_name, level=2)
        return levels

    @staticmethod
    def is_level(axis_type):
        if axis_type == Axis.Height or axis_type == Axis.Pressure or axis_type == Axis.GeoZ or axis_type == Axis.Hybrid:
            return True
        else:
            return False


def create_netcdf_first_guess_template(my_variables, my_nx, my_ny, fname="raw.nc"):

    if os.path.exists(fname):
        os.remove(fname)

    my_fg = netCDF4.Dataset(fname, "w")
    my_fg.createDimension("y", my_ny)
    my_fg.createDimension("x", my_nx)
    my_fg.createDimension("time", 1)
    my_fg.createVariable("time", "f8", "time")
    my_fg.variables["time"].long_name = "time"
    my_fg.variables["time"].standard_name = "time"
    my_fg.variables["time"].units = "seconds since 1970-01-01 00:00:00 +00:00"
    my_fg.createVariable("longitude", "f8", ("y", "x"))
    my_fg.variables["longitude"].units = "degree_east"
    my_fg.variables["longitude"].long_name = "longitude"
    my_fg.variables["longitude"].standard_name = "longitude"
    my_fg.createVariable("latitude", "f8", ("y", "x"))
    my_fg.variables["latitude"].units = "degree_north"
    my_fg.variables["latitude"].long_name = "latitude"
    my_fg.variables["latitude"].standard_name = "latitude"
    my_fg.createVariable("x", "f4", "x")
    my_fg.variables["x"].long_name = "x-coordinate in Cartesian system"
    my_fg.variables["x"].standard_name = "projection_x_coordinate"
    my_fg.variables["x"].units = "m"
    my_fg.createVariable("y", "f4", "y")
    my_fg.variables["y"].long_name = "y-coordinate in Cartesian system"
    my_fg.variables["y"].standard_name = "projection_y_coordinate"
    my_fg.variables["y"].units = "m"

    standard_name = {"air_temperature_2m": "air_temperature",
                     "relative_humidity_2m": "relative_humidity",
                     "altitude": "altitude",
                     "surface_snow_thickness": "surface_snow_thickness",
                     "land_area_fraction": "land_area_fraction"}
    long_name = {"air_temperature_2m": "Screen level temperature (T2M)",
                 "relative_humidity_2m": "Screen level relative humidity (RH2M)",
                 "altitude": "Altitude",
                 "surface_snow_thickness": "Surface snow thickness",
                 "land_area_fraction": "Land Area Fraction"}
    units = {"air_temperature_2m": "K",
             "relative_humidity_2m": "1",
             "altitude": "m",
             "surface_snow_thickness": "m",
             "land_area_fraction": "1"}
    fillvalue = {"air_temperature_2m": "9.96921e+36",
                 "relative_humidity_2m": "9.96921e+36",
                 "altitude": "9.96921e+36",
                 "surface_snow_thickness": "9.96921e+36",
                 "land_area_fraction": "9.96921e+36"}

    for my_var in my_variables:
        my_fg.createVariable(my_var, "f4", ("y", "x"), fill_value=fillvalue[my_var])
        my_fg.variables[my_var].long_name = long_name[my_var]
        my_fg.variables[my_var].standard_name = standard_name[my_var]
        my_fg.variables[my_var].units = units[my_var]

    return my_fg


def read_first_guess_netcdf_file(input_file, var):

    fh = netCDF4.Dataset(input_file)
    lons = fh["longitude"][:]
    lats = fh["latitude"][:]

    validtime = int(cfunits.Units.conform(fh["time"][:], cfunits.Units(fh["time"].units),
                    cfunits.Units("seconds since 1970-01-01 00:00:00")))
    validtime = datetime.fromtimestamp(validtime)

    nx = lons.shape[1]
    ny = lons.shape[0]

    lons = np.array(np.reshape(lons, [nx * ny], order="F"))
    lats = np.array(np.reshape(lats, [nx * ny], order="f"))

    # print(lons.shape, lats.shape, type(lons))
    geo = surfex.Geo(nx*ny, nx, ny, lons, lats)

    background = fh[var][:]
    background = np.array(np.reshape(background, [nx * ny]))
    background = np.reshape(background, [ny, nx])
    background = np.transpose(background)
    # background = background.tolist()
    # background = np.asarray(background)
    glafs = fh["land_area_fraction"][:]
    glafs = np.array(np.reshape(glafs, [nx * ny]))
    glafs = np.reshape(glafs, [ny, nx])
    glafs = np.transpose(glafs)
    # glafs = glafs.tolist()
    # glafs = np.asarray(glafs)
    gelevs = fh["altitude"][:]
    gelevs = np.array(np.reshape(gelevs, [nx * ny]))
    gelevs = np.reshape(gelevs, [ny, nx])
    gelevs = np.transpose(gelevs)
    # gelevs = gelevs.tolist()
    # gelevs = np.asarray(gelevs)

    fh.close()
    return geo, validtime, background, glafs, gelevs


def write_analysis_netcdf_file(filename, field, var, validtime, elevs, lafs, new_file=True, geo=None):

    fh = None
    if os.path.exists(filename):
        fh = netCDF4.Dataset(filename, mode="a")
    else:
        new_file = True

    if new_file:
        if geo is None:
            raise Exception("You need to provide geo to write a new file")
        fh = create_netcdf_first_guess_template([var, "altitude", "land_area_fraction"],
                                                geo.nlons, geo.nlats, fname=filename)
        fh.variables["longitude"][:] = np.transpose(geo.lons)
        fh.variables["latitude"][:] = np.transpose(geo.lats)
        fh.variables["x"][:] = [i for i in range(0, geo.nlons)]
        fh.variables["y"][:] = [i for i in range(0, geo.nlats)]
        fh.variables["altitude"][:] = np.transpose(elevs)
        fh.variables["land_area_fraction"][:] = np.transpose(lafs)

    fh["time"][:] = float(validtime.strftime("%s"))
    fh[var][:] = np.transpose(field)
    fh.close()


def oi2soda(dtg, t2m=None, rh2m=None, sd=None, output=None, debug=False):

    def check_input_to_soda_dimensions(my_nx, my_ny, nx1, ny1):

        if my_nx < 0:
            my_nx = nx1
        if my_ny < 0:
            my_ny = ny1
        if my_nx != nx1:
            raise Exception("Mismatch in nx dimension " + str(my_nx) + " != " + str(nx1))
        if my_ny != ny1:
            raise Exception("Mismatch in ny dimension " + str(my_ny) + " != " + str(ny1))

        return my_nx, my_ny

    yy = dtg.strftime("%y")
    mm = dtg.strftime("%m")
    dd = dtg.strftime("%d")
    hh = dtg.strftime("%H")
    nx = -1
    ny = -1
    i = 0

    t2m_var = None
    rh2m_var = None
    sd_var = None
    if t2m is not None:
        t2m_fh = netCDF4.Dataset(t2m["file"], "r")
        if debug:
            print(t2m["var"], t2m_fh.variables[t2m["var"]].shape)
        t2m_var = t2m_fh.variables[t2m["var"]][:]

        i = i + 1
        nx, ny = check_input_to_soda_dimensions(nx, ny, t2m_fh.variables[t2m["var"]].shape[1],
                                                t2m_fh.variables[t2m["var"]].shape[0])
        if debug:
            print(t2m_var.shape, nx*ny)
        t2m_var = np.reshape(t2m_var, ny * nx, order="F")
        mask = np.ma.is_masked(t2m_var)
        t2m_var[mask] = 999.
        t2m_var = t2m_var.tolist()

    if rh2m is not None:
        rh2m_fh = netCDF4.Dataset(rh2m["file"], "r")
        if debug:
            print(rh2m["var"], rh2m_fh.variables[rh2m["var"]].shape)

        i = i + 1
        nx, ny = check_input_to_soda_dimensions(nx, ny, rh2m_fh.variables[rh2m["var"]].shape[1],
                                                rh2m_fh.variables[rh2m["var"]].shape[0])
        rh2m_var = rh2m_fh.variables[rh2m["var"]][:]
        rh2m_var = rh2m_var.reshape([ny * nx], order="F")
        mask = np.ma.is_masked(rh2m_var)
        rh2m_var[mask] = 999.
        rh2m_var = rh2m_var.tolist()

    if sd is not None:
        sd_fh = netCDF4.Dataset(sd["file"], "r")
        if debug:
            print(sd["var"], sd_fh.variables[sd["var"]].shape)

        i = i + 1
        nx, ny = check_input_to_soda_dimensions(nx, ny, sd_fh.variables[sd["var"]].shape[1],
                                                sd_fh.variables[sd["var"]].shape[0])

        sd_var = sd_fh.variables[sd["var"]][:]
        sd_var = sd_var.reshape([ny * nx], order="F")
        mask = np.ma.is_masked(sd_var)
        sd_var[mask] = 999.
        sd_var = sd_var.tolist()

    if i == 0:
        raise Exception("You must specify at least one file to read from!")

    if t2m_var is None:
        t2m_var = [999] * (nx * ny)
    if rh2m_var is None:
        rh2m_var = [999] * (nx * ny)
    if sd_var is None:
        sd_var = [999] * (nx * ny)

    if output is None:
        out = open("OBSERVATIONS_" + str(yy) + str(mm) + str(dd) + "H" + str(hh)+".DAT", "w")
    else:
        out = open(output, "w")

    for i in range(0, nx*ny):
        out.write(str(t2m_var[i]) + " " + str(rh2m_var[i]) + " " + str(sd_var[i]) + "\n")
        if debug:
            print("i", i)

    out.close()



from datetime import datetime
from math import exp
import sys
import surfex
import numpy as np
try:
    import eccodes
except ImportError:
    eccodes = None
    print("ECCODES not found. Needed for bufr reading")
except RuntimeError:
    eccodes = None
    print("ECCODES not found. Needed for bufr reading")
# Needed in Python 3.5
except:
    eccodes = None
    gribapi = None


class BufrObservationSet(surfex.obs.ObservationSet):
    def __init__(self, bufrfile, variables, valid_dtg, valid_range, lonrange=None, latrange=None, debug=False,
                 label="bufr"):

        if lonrange is None:
            lonrange = [-180, 180]
        if latrange is None:
            latrange = [-90, 90]

        if eccodes is None:
            raise Exception("ECCODES not found. Needed for bufr reading")

        # open bufr file
        f = open(bufrfile)

        # define the keys to be printed
        keys = [
            # 'blockNumber',
            # 'stationNumber',
            'latitude',
            'longitude',
            'year',
            'month',
            'day',
            'hour',
            'minute',
            'heightOfStationGroundAboveMeanSeaLevel',
            'heightOfStation',
            'stationNumber',
            'blockNumber'
        ]
        nerror = {}
        ntime = {}
        nundef = {}
        ndomain = {}
        nobs = {}
        for var in variables:
            if var == "relativeHumidityAt2M":
                keys.append("airTemperatureAt2M")
                keys.append("dewpointTemperatureAt2M")
            else:
                keys.append(var)
            nerror.update({var: 0})
            ntime.update({var: 0})
            nundef.update({var: 0})
            ndomain.update({var: 0})
            nobs.update({var: 0})

        # The cloud information is stored in several blocks in the
        # SYNOP message and the same key means a different thing in different
        # parts of the message. In this example we will read the first
        # cloud block introduced by the key
        # verticalSignificanceSurfaceObservations=1.
        # We know that this is the first occurrence of the keys we want to
        # read so in the list above we used the # (occurrence) operator
        # accordingly.

        print("Reading " + bufrfile)
        print("Looking for keys: " + str(keys))
        cnt = 0
        observations = list()

        # loop for the messages in the file
        # nerror = 0
        # ndomain = 0
        # nundef = 0
        # ntime = 0
        not_decoded = 0
        # removed = 0
        while 1:
            # get handle for message
            bufr = eccodes.codes_bufr_new_from_file(f)
            if bufr is None:
                break

            # print("message: %s" % cnt)

            # we need to instruct ecCodes to expand all the descriptors
            # i.e. unpack the data values
            try:
                eccodes.codes_set(bufr, 'unpack', 1)
                decoded = True
            except eccodes.CodesInternalError as err:
                not_decoded = not_decoded + 1
                print('Error with key="unpack" : %s' % err.msg)
                decoded = False

            # print the values for the selected keys from the message
            if decoded:
                lat = np.nan
                lon = np.nan
                value = np.nan
                elev = np.nan
                year = -1
                month = -1
                day = -1
                hour = -1
                minute = -1
                stid = "NA"
                station_number = -1
                block_number = -1
                t2m = np.nan
                td2m = np.nan
                # rh2m = np.nan
                sd = np.nan
                # all_found = True
                for key in keys:
                    try:
                        val = eccodes.codes_get(bufr, key)
                        # if val != CODES_MISSING_DOUBLE:
                        #    print('  %s: %s' % (key,val))
                        if val == eccodes.CODES_MISSING_DOUBLE or val == eccodes.CODES_MISSING_LONG:
                            val = np.nan
                        if key == "latitude":
                            lat = val
                        if key == "longitude":
                            lon = val
                        if key == "year":
                            year = val
                        if key == "month":
                            month = val
                        if key == "day":
                            day = val
                        if key == "hour":
                            hour = val
                        if key == "minute":
                            minute = val
                        if key == "heightOfStation":
                            elev = val
                        if key == "heightOfStationGroundAboveMeanSeaLevel":
                            elev = val
                        if key == "stationNumber":
                            station_number = val
                        if key == "blockNumber":
                            block_number = val
                        if key == "airTemperatureAt2M":
                            t2m = val
                        if key == "dewpointTemperatureAt2M":
                            td2m = val
                        if key == "totalSnowDepth":
                            sd = val

                    except eccodes.CodesInternalError:
                        pass
                        # all_found = False
                        # print('Report does not contain key="%s" : %s' % (key, err.msg))

                # Assign value to var
                for var in variables:
                    if var == "relativeHumidityAt2M":
                        if not np.isnan(t2m) and not np.isnan(td2m):
                            value = self.td2rh(td2m, t2m)
                        value = value * 0.01
                    elif var == "airTemperatureAt2M":
                        value = t2m
                    elif var == "totalSnowDepth":
                        value = sd
                    else:
                        raise NotImplementedError("Var " + var + " is not coded! Please do it!")

                    all_found = True
                    if np.isnan(lat):
                        all_found = False
                    if np.isnan(lon):
                        all_found = False
                    if year == -1:
                        all_found = False
                    if month == -1:
                        all_found = False
                    if day == -1:
                        all_found = False
                    if hour == -1:
                        all_found = False
                    if minute == -1:
                        all_found = False
                    if np.isnan(elev):
                        all_found = False
                    if np.isnan(value):
                        all_found = False

                    if not all_found:
                        nerror.update({var: nerror[var] + 1})

                    # print(lon, lonrange[0], lonrange[1], lat, latrange[0],latrange[1])
                    if latrange[0] <= lat <= latrange[1] and lonrange[0] <= lon <= lonrange[1]:
                        obs_dtg = datetime(year=year, month=month, day=day, hour=hour, minute=minute)
                        # print(value)
                        if not np.isnan(value):
                            if self.inside_window(obs_dtg, valid_dtg, valid_range):
                                # print(valid_dtg, lon, lat, value, elev, stid)
                                if station_number > 0 and block_number > 0:
                                    stid = str((block_number * 1000) + station_number)
                                observations.append(surfex.obs.Observation(obs_dtg, lon, lat, value,
                                                                           elev=elev, stid=stid, varname=var))
                                nobs.update({var: nobs[var] + 1})
                            else:
                                ntime.update({var: ntime[var] + 1})
                        else:
                            nundef.update({var: nundef[var] + 1})
                    else:
                        ndomain.update({var: ndomain[var] + 1})

                cnt += 1

                if (cnt % 1000) == 0:
                    print('.', end='')
                    sys.stdout.flush()

            # delete handle
            eccodes.codes_release(bufr)

        print("\nFound " + str(len(observations)) + "/" + str(cnt))
        print("Not decoded: " + str(not_decoded))
        for var in variables:
            print("\nObservations for var=" + var + ": " + str(nobs[var]))
            print("Observations removed because of domain check: " + str(ndomain[var]))
            print("Observations removed because of not being defined/found: " + str(nundef[var]))
            print("Observations removed because of time window: " + str(ntime[var]))
            print("Messages not containing information on all keys: " + str(nerror[var]))
        # close the file
        f.close()

        surfex.obs.ObservationSet.__init__(self, observations, debug=debug, label=label)

    @staticmethod
    def td2rh(td, t, kelvin=True):
        if kelvin:
            if td < 100:
                raise Exception("Dew point temperature is probably not Kelvin")
            if t < 100:
                raise Exception("Temperature is probably not Kelvin")
            td = td - 273.15
            t = t - 273.15

        rh = 100 * (exp((17.625 * td) / (243.04 + td)) / exp((17.625 * t) / (243.04 + t)))
        if rh > 110 or rh < 1:
            print("\nWARNING: Calculated rh to " + str(rh) + " from " + str(td) + " and " + str(t) +
                  ". Set it to missing")
            rh = np.nan
        elif rh > 100:
            print("\nWARNING: Calculated rh to " + str(rh) + " from " + str(td) + " and " + str(t) +
                  ". Truncate to 100%")
            rh = 100
        return rh

    @staticmethod
    def inside_window(obs_dtg, valid_dtg, valid_range):
        if valid_dtg is None:
            return True
        else:
            if (valid_dtg - valid_range) <= obs_dtg <= (valid_dtg + valid_range):
                return True
            else:
                return False



import unittest
import surfex
from datetime import datetime
import json
import os


class GribTest(unittest.TestCase):

    def setUp(self):
        self.testdata = "testdata/"
        self.rootdir = os.path.abspath(os.curdir)
        domain = surfex.geo.set_domain(json.load(open(self.rootdir + "/test/settings/domains.json", "r")),
                                       "CONF_PROJ_TEST")
        self.geo = surfex.geo.get_geo_object(domain)
        self.converter = "none"
        self.config = {
            "grib1": {
                "fcint": 10800,
                "file_inc": 3600,
                "offset": 0
            },
            "grib2": {
                "fcint": 10800,
                "file_inc": 3600,
                "offset": 0
            },
            "t2m": {
                "grib1": {
                    "converter": {
                        "none": {
                            "parameter": 11,
                            "type": 105,
                            "level": 2,
                            "tri": 0,
                            "filepattern": self.testdata + "/fc2020111303+0003grib1"
                        }
                    }
                }
            },
            "t1": {
                "grib2": {
                    "converter": {
                        "none": {
                            "discipline": 0,
                            "parameterCategory": 0,
                            "parameterNumber": 0,
                            "levelType": 103,
                            "typeOfStatisticalProcessing": -1,
                            "level": 2,
                            "filepattern": self.testdata + "/fc2020111303+0003grib2"
                        }
                    }
                }
            },
        }

    def test_grib1_from_converter(self):

        # Grib 1
        fileformat = "grib1"
        var = "t2m"
        print(var, fileformat)
        defs = self.config[fileformat]
        converter_conf = self.config[var][fileformat]["converter"]
        debug = True

        validtime = datetime(year=2020, month=3, day=30, hour=6)
        cache = surfex.Cache(debug, 7200)
        converter = surfex.read.Converter(self.converter, validtime, defs, converter_conf, fileformat, validtime,
                                          debug=debug)
        surfex.read.ConvertedInput(self.geo, var, converter).read_time_step(validtime, cache)

    def test_grib2_from_converter(self):
        fileformat = "grib2"
        var = "t1"
        print(var, fileformat)
        defs = self.config[fileformat]
        converter_conf = self.config[var][fileformat]["converter"]
        debug = True

        validtime = datetime(year=2020, month=3, day=30, hour=6)
        cache = surfex.Cache(debug, 7200)
        converter = surfex.read.Converter(self.converter, validtime, defs, converter_conf, fileformat, validtime,
                                          debug=debug)
        surfex.read.ConvertedInput(self.geo, var, converter).read_time_step(validtime, cache)





import unittest
import surfex
import json
import os


class TitanTest(unittest.TestCase):

    def setUp(self):
        self.testdata = "testdata/"
        self.qc_settings = {
            "t2m": {
                "do_test": True,
                "plausibility": {
                    "minval": 200,
                    "maxval": 350
                },
                "firstguess": {
                    "fg_file": self.testdata + "/unittest_FirstGuess4gridpp_grib2.nc",
                    "fg_var": "air_temperature_2m",
                    "negdiff": 2,
                    "posdiff": 3,
                    "do_test": False
                },
                # Not working yet
                "buddy": {
                    "do_test": False
                },
                "climatology": {
                    "do_test": False,
                    "minval": 270,
                    "maxval": 275
                },
                "sct": {
                },
                "redundancy": {
                },
                "blacklist": {
                },
                "domain": {
                },
                "nometa": {
                },
                "fraction": {
                    "fraction_file": self.testdata + "/unittest_FirstGuess4gridpp_grib2.nc",
                    "fraction_var": "land_area_fraction",
                },
                "sets": {
                    "bufr": {
                        "filepattern": "testdata_obs/bufr_t2m.json",
                        "filetype": "json",
                        "varname": "airTemperatureAt2M",
                        "tests": {
                            "firstguess": {
                                "do_test": True
                            }
                        }
                    },
                    "netatmo": {
                        "filepattern": self.testdata + "/unittest_netatmo_t2m.json",
                        "varname": "Temperature",
                        "filetype": "json",
                        "tests": {
                            "firstguess": {
                            }
                        }
                    },
                    "frost": {
                        "filepattern": self.testdata + "/unittest_frost_t2m.json",
                        "varname": "air_temperature",
                        "filetype": "json",
                        "tests": {
                            "firstguess": {
                                "do_test": True
                            }
                        }
                    }
                }
            },
            "rh2m": {
                "do_test": True,
                "plausibility": {
                    "minval": 0,
                    "maxval": 1
                },
                "firstguess": {
                    "fg_file": self.testdata + "/unittest_FirstGuess4gridpp_grib1.nc",
                    "fg_var": "relative_humidity_2m",
                    "negdiff": 0.2,
                    "posdiff": 0.2,
                    "do_test": False
                },
                # Not working yet
                "buddy": {
                    "do_test": False
                },
                "climatology": {
                    "do_test": False,
                    "minval": 0,
                    "maxval": 1
                },
                "sct": {
                },
                "redundancy": {
                },
                "blacklist": {
                },
                "domain": {
                },
                "nometa": {
                },
                "fraction": {
                    "fraction_file": self.testdata + "/unittest_FirstGuess4gridpp_grib1.nc",
                    "fraction_var": "land_area_fraction",
                },
                "sets": {
                    "bufr": {
                        "filepattern": self.testdata + "/unittest_bufr_rh2m.json",
                        "filetype": "json",
                        "varname": "relativeHumidityAt2M",
                        "tests": {
                            "firstguess": {
                                "do_test": True
                            }
                        }
                    },
                    "netatmo": {
                        "filepattern": self.testdata + "/unittest_netatmo_rh2m.json",
                        "varname": "Humidity",
                        "filetype": "json",
                        "tests": {
                            "firstguess": {
                            }
                        }
                    },
                    "frost": {
                        "filepattern": self.testdata + "/unittest_frost_rh2m.json",
                        "varname": "relative_humidity",
                        "filetype": "json",
                        "tests": {
                            "firstguess": {
                                "do_test": True
                            }
                        }
                    }
                }
            },
            "sd": {
                "do_test": True,
                "plausibility": {
                    "minval": 0,
                    "maxval": 50
                },
                "firstguess": {
                    "fg_file": self.testdata + "/unittest_FirstGuess4gridpp_grib2.nc",
                    "fg_var": "surface_snow_thickness",
                    "negdiff": 0.4,
                    "posdiff": 0.4,
                    "do_test": True
                },
                # Not working yet
                "buddy": {
                    "do_test": False
                },
                "climatology": {
                    "do_test": False,
                    "minval": 0,
                    "maxval": 1
                },
                "sct": {
                },
                "redundancy": {
                },
                "blacklist": {
                },
                "domain": {
                },
                "nometa": {
                },
                "fraction": {
                    "fraction_file": self.testdata + "/unittest_FirstGuess4gridpp_grib2.nc",
                    "fraction_var": "land_area_fraction",
                },
                "sets": {
                    "bufr": {
                        "filepattern": self.testdata + "/unittest_bufr_sd.json",
                        "filetype": "json",
                        "varname": "totalSnowDepth",
                        "tests": {
                            "firstguess": {
                                "do_test": True
                            }
                        }
                    },
                    "frost": {
                        "filepattern": self.testdata + "/unittest_frost_sd.json",
                        "varname": "surface_snow_thickness",
                        "filetype": "json",
                        "tests": {
                            "firstguess": {
                                "do_test": True
                            }
                        }
                    }
                }
            }
        }

    def test_titan_t2m(self):
        json.dump(self.qc_settings, open("unittest_qc_settings_t2m.json", "w"))
        json.dump({}, open("unittest_blacklist_t2m.json", "w"))
        argv = [
            "-i", "unittest_qc_settings_t2m.json",
            "-v", "t2m",
            "-dtg", "2020033006",
            "--blacklist", "unittest_blacklist_t2m.json",
            "--domain", "test/settings/conf_proj_test.json",
            "-o", "unittest_qc_t2m.json",
            "domain", "blacklist", "nometa", "plausibility", "redundancy", "firstguess", "fraction",
            "buddy", "climatology", "sct"
        ]
        kwargs = surfex.parse_args_titan(argv)
        surfex.run_titan(**kwargs)

    def test_titan_t2m_harmonie(self):
        json.dump(self.qc_settings, open("unittest_qc_settings_t2m_hm.json", "w"))
        json.dump({}, open("unittest_blacklist_t2m_hm.json", "w"))
        argv = [
            "-i", "unittest_qc_settings_t2m_hm.json",
            "-v", "t2m",
            "-dtg", "2020033006",
            "--harmonie",
            "--blacklist", "unittest_blacklist_t2m_hm.json",
            "-o", "unittest_qc_t2m_hm.json",
            "domain", "blacklist", "nometa", "plausibility", "redundancy", "firstguess", "fraction",
            "buddy", "climatology", "sct"
        ]
        env = json.load(open("test/settings/hm_env.json", "r"))
        os.environ.update(env)
        kwargs = surfex.parse_args_titan(argv)
        surfex.run_titan(**kwargs)

    def test_titan_rh2m(self):
        json.dump(self.qc_settings, open("unittest_qc_settings_rh2m.json", "w"))
        json.dump({}, open("unittest_blacklist_rh2m.json", "w"))
        argv = [
            "-i", "unittest_qc_settings_rh2m.json",
            "-v", "rh2m",
            "-dtg", "2020033006",
            "--blacklist", "unittest_blacklist_rh2m.json",
            "--domain", "test/settings/conf_proj_test.json",
            "-o", "unittest_qc_rh2m.json",
            "domain", "blacklist", "nometa", "plausibility", "redundancy", "firstguess", "fraction",
            "buddy", "climatology", "sct"
        ]
        kwargs = surfex.parse_args_titan(argv)
        surfex.run_titan(**kwargs)

    def test_titan_rh2m_harmonie(self):
        json.dump(self.qc_settings, open("unittest_qc_settings_rh2m_hm.json", "w"))
        json.dump({}, open("unittest_blacklist_rh2m_hm.json", "w"))
        argv = [
            "-i", "unittest_qc_settings_rh2m_hm.json",
            "-v", "rh2m",
            "-dtg", "2020033006",
            "--harmonie",
            "--blacklist", "unittest_blacklist_rh2m_hm.json",
            "-o", "unittest_qc_rh2m_hm.json",
            "domain", "blacklist", "nometa", "plausibility", "redundancy", "firstguess", "fraction",
            "buddy", "climatology", "sct"
        ]
        env = json.load(open("test/settings/hm_env.json", "r"))
        os.environ.update(env)
        kwargs = surfex.parse_args_titan(argv)
        surfex.run_titan(**kwargs)

    def test_titan_sd(self):
        json.dump(self.qc_settings, open("unittest_qc_settings_sd.json", "w"))
        json.dump({}, open("unittest_blacklist_sd.json", "w"))
        argv = [
            "-i", "unittest_qc_settings_sd.json",
            "-v", "sd",
            "-dtg", "2020033006",
            "--blacklist", "unittest_blacklist_sd.json",
            "--domain", "test/settings/conf_proj_test.json",
            "-o", "unittest_qc_sd.json",
            "domain", "blacklist", "nometa", "plausibility", "redundancy", "firstguess", "fraction",
            "buddy", "climatology", "sct"
        ]
        kwargs = surfex.parse_args_titan(argv)
        surfex.run_titan(**kwargs)

    def test_titan_sd_harmonie(self):
        json.dump(self.qc_settings, open("unittest_qc_settings_sd_hm.json", "w"))
        json.dump({}, open("unittest_blacklist_sd_hm.json", "w"))
        argv = [
            "-i", "unittest_qc_settings_sd_hm.json",
            "-v", "sd",
            "-dtg", "2020033006",
            "--harmonie",
            "--blacklist", "unittest_blacklist_sd_hm.json",
            "-o", "unittest_qc_sd_hm.json",
            "domain", "blacklist", "nometa", "plausibility", "redundancy", "firstguess", "fraction",
            "buddy", "climatology", "sct"
        ]
        env = json.load(open("test/settings/hm_env.json", "r"))
        os.environ.update(env)
        kwargs = surfex.parse_args_titan(argv)
        surfex.run_titan(**kwargs)
